import 'package:flutter/material.dart';
import 'dart:math' as math;
import 'painter.dart';
import 'enum.dart';

class CustomCircleIndicator extends StatefulWidget {
  const CustomCircleIndicator({
    Key? key,
    this.value,
    this.size,
    this.checkOnFinish = false,
    this.checkColor,
    this.backColor,
    this.valueColor,
    this.indicatorType = IndicatorType.inside,
    this.pointerColor,
    this.showValue = true,
    this.valueTextStyle,
  }) : super(key: key);

  final double? value, size;
  final bool checkOnFinish, showValue;
  final Color? checkColor, backColor, valueColor, pointerColor;
  final IndicatorType indicatorType;
  final TextStyle? valueTextStyle;

  @override
  _CustomCircleIndicatorState createState() => _CustomCircleIndicatorState();
}

class _CustomCircleIndicatorState extends State<CustomCircleIndicator>
    with TickerProviderStateMixin {
  AnimationController? firstCircleController;
  AnimationController? secondCircleController;
  double? value, size;
  double screenWidth = 0, screenHeight = 0;
  bool animationsInitialized = false,
      checkOnFinish = false,
      check = false,
      showValue = true;
  double valueOpacity = 1;
  Color? checkColor, backColor, valueColor, pointerColor;
  IndicatorType indicatorType = IndicatorType.inside;
  TextStyle? valueTextStyle;

  void setValues() {
    value = widget.value;
    showValue = widget.showValue;
    indicatorType = widget.indicatorType;
    size = widget.size ?? screenWidth * 0.3;
    checkOnFinish = widget.checkOnFinish;
    checkColor = widget.checkColor ?? Colors.green;
    backColor = widget.backColor ?? Colors.black;
    valueColor = widget.valueColor ?? Colors.white;
    pointerColor = widget.pointerColor ?? Colors.black;
    valueTextStyle = widget.valueTextStyle;
  }

  void initializeAnimations() {
    if (value == null && !animationsInitialized) {
      animationsInitialized = true;
      firstCircleController = AnimationController(
        vsync: this,
        duration: const Duration(milliseconds: 500),
        lowerBound: 0,
        upperBound: math.pi * 2,
      );
      secondCircleController = AnimationController(
        vsync: this,
        duration: const Duration(milliseconds: 500),
        lowerBound: 0,
        upperBound: math.pi * 2,
      );

      firstCircleController!.addListener(() {
        setState(() {});
      });
      secondCircleController!.addListener(() {
        setState(() {});
      });

      firstCircleController!.addStatusListener((status) {
        if (status == AnimationStatus.completed) {
          Future.delayed(const Duration(milliseconds: 100)).then((value) {
            secondCircleController!.reset();
            secondCircleController!.forward();
          });
        }
      });
      secondCircleController!.addStatusListener((status) {
        if (status == AnimationStatus.completed) {
          Future.delayed(const Duration(milliseconds: 100)).then((value) {
            firstCircleController!.reset();
            secondCircleController!.reset();
            firstCircleController!.forward();
          });
        }
      });

      firstCircleController!.forward();
    }
  }

  @override
  void dispose() {
    if (firstCircleController != null) {
      firstCircleController!.dispose();
      secondCircleController!.dispose();
    }
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    screenHeight = MediaQuery.of(context).size.height;
    screenWidth = MediaQuery.of(context).size.width;
    setValues();
    initializeAnimations();
    if (value != null && value! > 100) {
      value = 100;
    }
    if (checkOnFinish && value == 100 && !check) {
      valueOpacity = 0;
      Future.delayed(const Duration(milliseconds: 550)).then((value) {
        setState(() {
          check = true;
          valueOpacity = 1;
        });
      });
    }
    if ((!checkOnFinish && check) || (value != 100 && check)) {
      valueOpacity = 0;
      Future.delayed(const Duration(milliseconds: 550)).then((value) {
        setState(() {
          check = false;
          valueOpacity = 1;
        });
      });
    }
    return Transform.rotate(
      angle: -math.pi / 2,
      child: Stack(
        children: [
          Center(
            child: SizedBox(
              height: size,
              width: size,
              child: CustomPaint(
                painter: DrawCircles(
                  firstCircleValue: value == null
                      ? firstCircleController!.value
                      : math.pi / 50 * value!,
                  secondCircleValue:
                      value == null ? secondCircleController!.value : 0,
                  backColor: backColor!,
                  valueColor: valueColor!,
                ),
                child: Container(),
              ),
            ),
          ),
          (value == null ||
                      !showValue ||
                      indicatorType == IndicatorType.pointer) &&
                  !check
              ? const SizedBox()
              : Transform.rotate(
                  angle: math.pi / 2,
                  child: Center(
                    child: AnimatedOpacity(
                      duration: const Duration(milliseconds: 500),
                      opacity: valueOpacity,
                      child: check
                          ? Icon(
                              Icons.check,
                              size: size! * 0.5,
                              color: checkColor,
                            )
                          : Text(
                              '${value! > 9 ? '${value!.toInt()}' : '0${value!.toInt()}'}%',
                              style: valueTextStyle ?? TextStyle(
                                fontSize: size! * 0.175,
                                fontWeight: FontWeight.bold,
                              ),
                            ),
                    ),
                  ),
                ),
          (value == 100 ||
                      value == null ||
                      indicatorType == IndicatorType.inside) &&
                  checkOnFinish
              ? const SizedBox()
              : value == null || !showValue
                  ? const SizedBox()
                  : indicatorType == IndicatorType.inside
                      ? const SizedBox()
                      : Center(
                          child: Transform.rotate(
                            angle: (math.pi / 2),
                            child: Transform.rotate(
                              angle: (value! / 100) * (math.pi * 2),
                              child: Container(
                                alignment: Alignment.topCenter,
                                height: size! - 10,
                                width: size! - 10,
                                decoration: const BoxDecoration(
                                  shape: BoxShape.circle,
                                ),
                                child: Column(
                                  children: [
                                    Transform.rotate(
                                      angle: math.pi,
                                      child: SizedBox(
                                        width: size! / 4,
                                        height: size! / 5,
                                        child: CustomPaint(
                                          painter: IndicatorPointer(
                                            color: pointerColor!,
                                          ),
                                          child: Container(),
                                        ),
                                      ),
                                    ),
                                    Text(
                                      '${value! > 9 ? '${value!.toInt()}' : '0${value!.toInt()}'}%',
                                      style: valueTextStyle ?? TextStyle(
                                        fontSize: size! * 0.11,
                                        fontWeight: FontWeight.bold,
                                      ),
                                    ),
                                  ],
                                ),
                              ),
                            ),
                          ),
                        ),
          // indicatorType == IndicatorType.pointer && value == 100
          //     ? Transform.rotate(
          //       angle: math.pi / 2,
          //       child: Center(
          //           child: Icon(
          //             Icons.check,
          //             size: size! * 0.5,
          //             color: checkColor,
          //           ),
          //         ),
          //     )
          //     : const SizedBox(),
        ],
      ),
    );
  }
}